@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 '''
 fsck_db.py - this file is part of S3QL (http://s3ql.googlecode.com)
 
@@ -7,9 +7,7 @@ Copyright (C)  Nikolaus Rath <Nikolaus@rath.org>
 This program can be distributed under the terms of the GNU GPLv3.
 '''
 
-from __future__ import division, print_function, absolute_import
 from argparse import ArgumentTypeError
-import logging
 import os
 import sys
 
@@ -20,11 +18,11 @@ if (os.path.exists(os.path.join(basedir, 'setup.py')) and
     os.path.exists(os.path.join(basedir, 'src', 's3ql', '__init__.py'))):
     sys.path = [os.path.join(basedir, 'src')] + sys.path
 
-from s3ql.common import setup_logging
+from s3ql.logging import logging, setup_logging
 from s3ql.fsck import ROFsck
 from s3ql.parse_args import ArgumentParser
 
-log = logging.getLogger("fsck")
+log = logging.getLogger(__name__)
 
 def parse_args(args):
 
