@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 '''
 pcp.py - this file is part of S3QL (http://s3ql.googlecode.com)
 
@@ -10,11 +10,8 @@ Copyright (C) 2010 Nikolaus Rath <Nikolaus@rath.org>
 This program can be distributed under the terms of the GNU GPLv3.
 '''
 
-from __future__ import division, print_function, absolute_import
-
 import sys
 import os
-import logging
 import subprocess
 
 # We are running from the S3QL source directory, make sure
@@ -24,10 +21,10 @@ if (os.path.exists(os.path.join(basedir, 'setup.py')) and
     os.path.exists(os.path.join(basedir, 'src', 's3ql', '__init__.py'))):
     sys.path = [os.path.join(basedir, 'src')] + sys.path
 
-from s3ql.common import (setup_logging)
+from s3ql.logging import logging, setup_logging
 from s3ql.parse_args import ArgumentParser
 
-log = logging.getLogger('pcp')
+log = logging.getLogger(__name__)
 
 def parse_args(args):
     '''Parse command line'''
@@ -97,4 +94,4 @@ def main(args=None):
 
 
 if __name__ == '__main__':
-    main(sys.argv[1:])
\ No newline at end of file
+    main(sys.argv[1:])
