@@ -10,7 +10,7 @@ from .logging import logging, QuietError, setup_logging
 from . import CURRENT_FS_REV, REV_VER_MAP
 from .backends.common import BetterBackend, get_backend, DanglingStorageURLError
 from .common import (get_backend_cachedir, get_seq_no, stream_write_bz2,
-                     stream_read_bz2, PICKLE_PROTOCOL)
+                     stream_read_bz2, PICKLE_PROTOCOL, is_mounted)
 from .metadata import restore_metadata, cycle_metadata, dump_metadata
 from .parse_args import ArgumentParser
 from datetime import datetime as Datetime
@@ -77,18 +77,15 @@ def main(args=None):
 
     # Check if fs is mounted on this computer
     # This is not foolproof but should prevent common mistakes
-    match = options.storage_url + ' /'
-    with open('/proc/mounts', 'r') as fh:
-        for line in fh:
-            if line.startswith(match):
-                raise QuietError('Can not work on mounted file system.')
+    if is_mounted(options.storage_url):
+        raise QuietError('Can not work on mounted file system.')
 
     if options.action == 'clear':
         try:
             backend = get_backend(options, plain=True)
             atexit.register(backend.close)
         except DanglingStorageURLError as exc:
-            raise QuietError(str(exc)) from None
+            raise QuietError(str(exc))
         return clear(backend,
                      get_backend_cachedir(options.storage_url, options.cachedir))
 
@@ -96,7 +93,7 @@ def main(args=None):
         backend = get_backend(options)
         atexit.register(backend.close)
     except DanglingStorageURLError as exc:
-        raise QuietError(str(exc)) from None
+        raise QuietError(str(exc))
 
     if options.action == 'upgrade':
         return upgrade(backend, get_backend_cachedir(options.storage_url,
@@ -186,8 +183,12 @@ def change_passphrase(backend):
 
     backend.passphrase = wrap_pw
     backend['s3ql_passphrase'] = data_pw
+    backend['s3ql_passphrase_bak1'] = data_pw
+    backend['s3ql_passphrase_bak2'] = data_pw
+    backend['s3ql_passphrase_bak3'] = data_pw
     backend.passphrase = data_pw
 
+    
 def clear(backend, cachepath):
     print('I am about to delete all data in %s.' % backend,
           'This includes any S3QL file systems as well as any other stored objects.',
