@@ -27,6 +27,7 @@ import http.client
 import lzma
 import os
 import pickle
+import io
 import re
 import socket
 import stat
@@ -60,6 +61,7 @@ def retry(method):
         self = a[0]
         interval = 1 / 50
         waited = 0
+        retries = 0
         while True:
             try:
                 return method(*a, **kw)
@@ -73,10 +75,19 @@ def retry(method):
                             self.__class__.__name__, method.__name__, exc)
                     raise
 
-                log.info('Encountered %s exception (%s), retrying call to %s.%s...',
-                          type(exc).__name__, exc, self.__class__.__name__, method.__name__)
+                retries += 1
+                if retries <= 2:
+                    log_fn = log.debug
+                elif retries <= 4:
+                    log_fn = log.info
+                else:
+                    log_fn = log.warning
+                    
+                log_fn('Encountered %s exception (%s), retrying call to %s.%s for the %d-th time...',
+                       type(exc).__name__, exc, self.__class__.__name__, method.__name__, retries)
 
                 if hasattr(exc, 'retry_after') and exc.retry_after:
+                    log.debug('retry_after is %.2f seconds', exc.retry_after)
                     interval = exc.retry_after
                 
             time.sleep(interval)
@@ -186,7 +197,7 @@ def is_temp_network_error(exc):
 
     if isinstance(exc, (http.client.IncompleteRead, socket.timeout,
                         ssl.SSLZeroReturnError, ConnectionError, TimeoutError,
-                        InterruptedError)):
+                        InterruptedError, ssl.SSLEOFError, ssl.SSLSyscallError)):
         return True
      
     # Server closed connection
@@ -665,7 +676,7 @@ class BetterBackend(AbstractBackend, metaclass=ABCDocstMeta):
             try:
                 buf = metadata['data']
             except KeyError:
-                raise MalformedObjectError() from None
+                raise MalformedObjectError()
             buf = b64decode(buf)
 
         if encrypted:
@@ -676,7 +687,7 @@ class BetterBackend(AbstractBackend, metaclass=ABCDocstMeta):
         except pickle.UnpicklingError as exc:
             if (isinstance(exc.args[0], str)
                 and exc.args[0].startswith('invalid load key')):
-                raise ChecksumError('Invalid metadata') from None
+                raise ChecksumError('Invalid metadata')
             raise
 
         return metadata
@@ -801,45 +812,6 @@ class BetterBackend(AbstractBackend, metaclass=ABCDocstMeta):
         self.backend.close()
 
 
-class AbstractInputFilter(object, metaclass=ABCMeta):
-    '''Process data while reading'''
-
-    def __init__(self):
-        super().__init__()
-        self.buffer = b''
-
-    def read(self, size=None):
-        '''Try to read *size* bytes
-        
-        If *None*, read until EOF.
-        '''
-
-        if size is None:
-            remaining = 1 << 31
-        else:
-            remaining = size - len(self.buffer)
-
-        while remaining > 0:
-            buf = self._read(BUFSIZE)
-            if not buf:
-                break
-            remaining -= len(buf)
-            self.buffer += buf
-
-        if size is None:
-            buf = self.buffer
-            self.buffer = b''
-        else:
-            buf = self.buffer[:size]
-            self.buffer = self.buffer[size:]
-
-        return buf
-
-    @abstractmethod
-    def _read(self, size):
-        '''Read roughly *size* bytes'''
-        pass
-
 class CompressFilter(object):
     '''Compress data while writing'''
 
@@ -865,9 +837,11 @@ class CompressFilter(object):
             self.obj_size += len(buf)
 
     def close(self):
+        assert not self.closed
         buf = self.compr.flush()
-        self.fh.write(buf)
-        self.obj_size += len(buf)
+        if buf:
+            self.fh.write(buf)
+            self.obj_size += len(buf)
         self.fh.close()
         self.closed = True
 
@@ -883,48 +857,76 @@ class CompressFilter(object):
             raise RuntimeError('Object must be closed first.')
         return self.obj_size
 
+class InputFilter(io.RawIOBase):
 
-class DecompressFilter(AbstractInputFilter):
+    def readable(self):
+        return True
+
+    def readinto(self, buf):
+        var = self.read(len(buf))
+        buf[:len(var)] = var
+        return var
+
+    def read(self, size=-1):
+        if size == -1:
+            return self.readall()
+        elif size == 0:
+            return b''
+
+        b = bytearray(size)
+        len_ = self.readinto(b)
+        return b[:len_]
+    
+class DecompressFilter(InputFilter):
     '''Decompress data while reading'''
 
     def __init__(self, fh, decomp, metadata=None):
         '''Initialize
         
-        *fh* should be a file-like object. *decomp* should be a
-        fresh decompressor instance with a *decompress* method.
+        *fh* should be a file-like object and may be unbuffered. *decomp* should
+        be a fresh decompressor instance with a *decompress* method.
         '''
         super().__init__()
 
         self.fh = fh
         self.decomp = decomp
         self.metadata = metadata
+            
+    def read(self, size=-1):
+        '''Read up to *size* bytes
 
-    def _read(self, size):
-        '''Read roughly *size* bytes'''
+        This method is currently buggy and may also return *more*
+        than *size* bytes. Callers should be prepared to handle
+        that. This is because some of the used (de)compression modules
+        don't support output limiting.
+        '''
 
+        if size == -1:
+            return self.readall()
+        elif size == 0:
+            return b''
+        
         buf = b''
         while not buf:
             buf = self.fh.read(size)
             if not buf:
+                if not self.decomp.eof:
+                    raise ChecksumError('Premature end of stream.')
                 if self.decomp.unused_data:
                     raise ChecksumError('Data after end of compressed stream')
                 return b''
 
             try:
-                buf = self.decomp.decompress(buf)
-            except IOError as exc:
-                if exc.args[0].lower().startswith('invalid data stream'):
-                    raise ChecksumError('Invalid compressed stream') from None
+                buf = decompress(self.decomp, buf)
+            except ChecksumError:
+                # Still read the stream completely (so that in case of
+                # an encrypted stream we check the HMAC).
+                while True:
+                    buf = self.fh.read(BUFSIZE)
+                    if not buf:
+                        break
                 raise
-            except lzma.LZMAError as exc:
-                if exc.args[0].lower().startswith('corrupt input data'):
-                    raise ChecksumError('Invalid compressed stream') from None
-                raise
-            except zlib.error as exc:
-                if exc.args[0].lower().startswith('error -3 while decompressing'):
-                    raise ChecksumError('Invalid compressed stream') from None
-                raise
-
+            
         return buf
 
     def close(self):
@@ -937,6 +939,7 @@ class DecompressFilter(AbstractInputFilter):
         self.close()
         return False
 
+
 class EncryptFilter(object):
     '''Encrypt data while writing'''
 
@@ -976,18 +979,22 @@ class EncryptFilter(object):
 
         buf = struct.pack(b'<I', len(data)) + data
         self.hmac.update(buf)
-        buf = self.cipher.encrypt(buf)
-        if buf:
-            self.fh.write(buf)
-            self.obj_size += len(buf)
+        buf2 = self.cipher.encrypt(buf)
+        assert len(buf2) == len(buf)
+        self.fh.write(buf2)
+        self.obj_size += len(buf2)
 
     def close(self):
+        assert not self.closed
+        
         # Packet length of 0 indicates end of stream, only HMAC follows
         buf = struct.pack(b'<I', 0)
         self.hmac.update(buf)
-        buf = self.cipher.encrypt(buf + self.hmac.digest())
-        self.fh.write(buf)
-        self.obj_size += len(buf)
+        buf += self.hmac.digest()
+        buf2 = self.cipher.encrypt(buf)
+        assert len(buf) == len(buf2)
+        self.fh.write(buf2)
+        self.obj_size += len(buf2)
         self.fh.close()
         self.closed = True
 
@@ -1004,7 +1011,7 @@ class EncryptFilter(object):
         return self.obj_size
 
 
-class DecryptFilter(AbstractInputFilter):
+class DecryptFilter(InputFilter):
     '''Decrypt data while reading
     
     Reader has to read the entire stream in order for HMAC
@@ -1014,7 +1021,7 @@ class DecryptFilter(AbstractInputFilter):
     def __init__(self, fh, passphrase, metadata=None):
         '''Initialize
         
-        *fh* should be a file-like object.
+        *fh* should be a file-like object that may be unbuffered.
         '''
         super().__init__()
 
@@ -1023,7 +1030,7 @@ class DecryptFilter(AbstractInputFilter):
         self.remaining = 0 # Remaining length of current packet
         self.metadata = metadata
         self.hmac_checked = False
-
+            
         # Read nonce
         len_ = struct.unpack(b'<B', fh.read(struct.calcsize(b'<B')))[0]
         nonce = fh.read(len_)
@@ -1032,39 +1039,84 @@ class DecryptFilter(AbstractInputFilter):
         self.cipher = aes_cipher(key)
         self.hmac = hmac.new(key, digestmod=hashlib.sha256)
 
-    def _read(self, size):
-        '''Read roughly *size* bytes'''
+    def _read_and_decrypt(self, size):
+        '''Read and decrypt up to *size* bytes'''
 
+        if size <= 0:
+            raise ValueError("Exact *size* required (got %d)" % size)
+        
         buf = self.fh.read(size)
         if not buf:
-            if not self.hmac_checked:
-                raise ChecksumError('HMAC mismatch')
+            if self.hmac_checked:
+                return b''
+            else:
+                raise ChecksumError('Premature end of stream.')
+
+        # Work around https://bugs.launchpad.net/pycrypto/+bug/1256172
+        # cipher.decrypt refuses to work with anything but bytes
+        if not isinstance(buf, bytes):
+            buf = bytes(buf)
+            
+        len_ = len(buf)
+        buf = self.cipher.decrypt(buf)
+        assert len(buf) == len_
+        
+        return buf
+    
+    def read(self, size=-1):
+        '''Read up to *size* bytes'''
+
+        if size == -1:
+            return self.readall()
+        elif size == 0:
             return b''
-
-        inbuf = self.cipher.decrypt(buf)
+        
         outbuf = b''
+        inbuf = b''
         while True:
-
-            if len(inbuf) <= self.remaining:
+            
+            # If all remaining data is part of the same packet, return it.
+            if inbuf and len(inbuf) <= self.remaining:
                 self.remaining -= len(inbuf)
                 self.hmac.update(inbuf)
                 outbuf += inbuf
                 break
-            elif len(inbuf) <= self.remaining + self.off_size:
-                inbuf += self.fh.read(self.off_size)
+            
+            # Otherwise keep reading until we have something to return
+            # but make sure not to stop in packet header (so that we don't
+            # cache the partially read header from one invocation to the next).
+            to_next = self.remaining + self.off_size
+            if (not inbuf or len(inbuf) < to_next):
+                if not inbuf:
+                    buf = self._read_and_decrypt(size - len(outbuf))
+                    if not buf:
+                        break
+                else:
+                    buf = self._read_and_decrypt(to_next - len(inbuf))
+                    assert buf
+                inbuf += buf
                 continue
-                
+            
+            # Copy rest of current packet to output and start reading
+            # from next packet
             outbuf += inbuf[:self.remaining]
-            self.hmac.update(inbuf[:self.remaining + self.off_size])
-            paket_size = struct.unpack(b'<I', inbuf[self.remaining
-                                                    :self.remaining + self.off_size])[0]
-            inbuf = inbuf[self.remaining + self.off_size:]
+            self.hmac.update(inbuf[:to_next])
+            paket_size = struct.unpack(b'<I', inbuf[self.remaining:to_next])[0]
+            inbuf = inbuf[to_next:]
             self.remaining = paket_size
 
             # End of file, read and check HMAC
             if paket_size == 0:
-                if len(inbuf) != HMAC_SIZE:
-                    inbuf += self.cipher.decrypt(self.fh.read(HMAC_SIZE - len(inbuf)))
+                while len(inbuf) < HMAC_SIZE:
+                    # Don't read exactly the missing amount, we wan't to detect
+                    # if there's extraneous data
+                    buf = self._read_and_decrypt(HMAC_SIZE)
+                    assert buf
+                    inbuf += buf
+                
+                if len(inbuf) > HMAC_SIZE or self.fh.read(1):
+                    raise ChecksumError('Extraneous data at end of object')
+                
                 if not hmac.compare_digest(inbuf, self.hmac.digest()):
                     raise ChecksumError('HMAC mismatch')
                 self.hmac_checked = True
@@ -1082,7 +1134,7 @@ class DecryptFilter(AbstractInputFilter):
         self.close()
         return False
 
-class LegacyDecryptDecompressFilter(AbstractInputFilter):
+class LegacyDecryptDecompressFilter(io.RawIOBase):
     '''Decrypt and Decompress data while reading
     
     Reader has to read the entire stream in order for HMAC
@@ -1092,7 +1144,7 @@ class LegacyDecryptDecompressFilter(AbstractInputFilter):
     def __init__(self, fh, passphrase, decomp, metadata=None):
         '''Initialize
         
-        *fh* should be a file-like object.
+        *fh* should be a file-like object and may be unbuffered.
         '''
         super().__init__()
 
@@ -1110,14 +1162,36 @@ class LegacyDecryptDecompressFilter(AbstractInputFilter):
         self.cipher = aes_cipher(key)
         self.hmac = hmac.new(key, digestmod=hashlib.sha256)
 
-    def _read(self, size):
-        '''Read roughly *size* bytes'''
+    def _decrypt(self, buf):
+        # Work around https://bugs.launchpad.net/pycrypto/+bug/1256172
+        # cipher.decrypt refuses to work with anything but bytes
+        if not isinstance(buf, bytes):
+            buf = bytes(buf)
+            
+        len_ = len(buf)
+        buf = self.cipher.decrypt(buf)
+        assert len(buf) == len_
+        return buf
+    
+    def read(self, size=-1):
+        '''Read up to *size* bytes
+        
+        This method is currently buggy and may also return *more*
+        than *size* bytes. Callers should be prepared to handle
+        that. This is because some of the used (de)compression modules
+        don't support output limiting.
+        '''
+
+        if size == -1:
+            return self.readall()
+        elif size == 0:
+            return b''
 
         buf = None
         while not buf:
             buf = self.fh.read(size)
             if not buf and not self.hmac_checked:
-                if not hmac.compare_digest(self.cipher.decrypt(self.hash), 
+                if not hmac.compare_digest(self._decrypt(self.hash), 
                                            self.hmac.digest()):
                     raise ChecksumError('HMAC mismatch')
                 elif self.decomp and self.decomp.unused_data:
@@ -1128,24 +1202,11 @@ class LegacyDecryptDecompressFilter(AbstractInputFilter):
             elif not buf:
                 return b''
 
-            buf = self.cipher.decrypt(buf)
+            buf = self._decrypt(buf)
             if not self.decomp:
                 break
 
-            try:
-                buf = self.decomp.decompress(buf)
-            except IOError as exc:
-                if exc.args[0].lower().startswith('invalid data stream'):
-                    raise ChecksumError('Invalid compressed stream') from None
-                raise
-            except lzma.LZMAError as exc:
-                if exc.args[0].lower().startswith('corrupt input data'):
-                    raise ChecksumError('Invalid compressed stream') from None
-                raise
-            except zlib.error as exc:
-                if exc.args[0].lower().startswith('error -3 while decompressing'):
-                    raise ChecksumError('Invalid compressed stream') from None
-                raise
+            buf = decompress(self.decomp, buf)
 
         self.hmac.update(buf)
         return buf
@@ -1159,7 +1220,31 @@ class LegacyDecryptDecompressFilter(AbstractInputFilter):
     def __exit__(self, *a):
         self.close()
         return False
-    
+
+
+def decompress(decomp, buf):
+    '''Decompress *buf* using *decomp*
+
+    This method encapsulates exception handling for different
+    decompressors.
+    '''
+
+    try:
+        return decomp.decompress(buf)
+    except IOError as exc:
+        if exc.args[0].lower().startswith('invalid data stream'):
+            raise ChecksumError('Invalid compressed stream')
+        raise
+    except lzma.LZMAError as exc:
+        if exc.args[0].lower().startswith('corrupt input data'):
+            raise ChecksumError('Invalid compressed stream')
+        raise
+    except zlib.error as exc:
+        if exc.args[0].lower().startswith('error -3 while decompressing'):
+            raise ChecksumError('Invalid compressed stream')
+        raise
+
+
 def encrypt(buf, passphrase, nonce):
     '''Encrypt *buf*'''
 
@@ -1336,7 +1421,7 @@ def get_backend_factory(options, plain=False):
     try:
         __import__(backend_name)
     except ImportError:
-        raise QuietError('No such backend: %s' % hit.group(1)) from None
+        raise QuietError('No such backend: %s' % hit.group(1))
 
     backend_class = getattr(sys.modules[backend_name], 'Backend')
 
@@ -1394,13 +1479,13 @@ def get_backend_factory(options, plain=False):
         backend.fetch('s3ql_passphrase')
         
     except DanglingStorageURLError as exc:
-        raise QuietError(str(exc)) from None
+        raise QuietError(str(exc))
     
     except AuthorizationError:
-        raise QuietError('No permission to access backend.') from None
+        raise QuietError('No permission to access backend.')
 
     except AuthenticationError:
-        raise QuietError('Invalid credentials (or skewed system clock?).') from None
+        raise QuietError('Invalid credentials (or skewed system clock?).')
         
     except NoSuchObject:
         encrypted = False
@@ -1441,7 +1526,7 @@ def get_backend_factory(options, plain=False):
     try:
         data_pw = tmp_backend['s3ql_passphrase']
     except ChecksumError:
-        raise QuietError('Wrong file system passphrase') from None
+        raise QuietError('Wrong file system passphrase')
     finally:
         tmp_backend.close()
         
