@@ -136,7 +136,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             if force:
                 pass
             else:
-                raise NoSuchObject(key) from None
+                raise NoSuchObject(key)
 
     @retry_generator
     @copy_ancestor_docstring
@@ -196,7 +196,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             assert resp.length == 0
         except HTTPError as exc:
             if exc.status == 404:
-                raise NoSuchObject(key) from None
+                raise NoSuchObject(key)
             else:
                 raise
 
@@ -212,7 +212,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             assert resp.length == 0
         except HTTPError as exc:
             if exc.status == 404:
-                raise NoSuchObject(key) from None
+                raise NoSuchObject(key)
             else:
                 raise
 
@@ -228,7 +228,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
         try:
             resp = self._do_request('GET', '/%s%s' % (self.prefix, key))
         except NoSuchKeyError:
-            raise NoSuchObject(key) from None
+            raise NoSuchObject(key)
 
         return ObjectR(key, resp, self, extractmeta(resp))
 
@@ -269,7 +269,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             # Discard response body
             resp.read()
         except NoSuchKeyError:
-            raise NoSuchObject(src) from None
+            raise NoSuchObject(src)
 
     def _do_request(self, method, path, subres=None, query_string=None,
                     headers=None, body=None):
@@ -603,13 +603,16 @@ class ObjectR(object):
         self.md5 = hashlib.md5()
 
     def read(self, size=None):
-        '''Read object data
+        '''Read up to *size* bytes of object data
         
         For integrity checking to work, this method has to be called until
         it returns an empty string, indicating that all data has been read
         (and verified).
         '''
 
+        if size == 0:
+            return b''
+
         # chunked encoding handled by httplib
         buf = self.resp.read(size)
 
@@ -698,6 +701,8 @@ class ObjectW(object):
 
         log.debug('ObjectW(%s).close(): start', self.key)
 
+        assert not self.closed
+
         self.fh.seek(0)
         self.headers['Content-Length'] = self.obj_size
         self.headers['Content-Type'] = 'application/octet-stream'
@@ -706,14 +711,13 @@ class ObjectW(object):
         etag = resp.getheader('ETag').strip('"')
         assert resp.length == 0
 
-        # Try-catch to ensure that we don't loose the BadDigest exception
-        try:
-            if etag != self.md5.hexdigest():
+        if etag != self.md5.hexdigest():
+            # delete may fail, but we don't want to loose the BadDigest exception
+            try:
+                self.backend.delete(self.key)
+            finally:
                 raise BadDigestError('BadDigest', 'MD5 mismatch for %s (received: %s, sent: %s)' %
                                      (self.key, etag, self.md5.hexdigest))
-        except:
-            self.backend.delete(self.key)
-            raise
 
         self.fh.close()
         self.closed = True
@@ -778,7 +782,7 @@ def extractmeta(resp):
         except pickle.UnpicklingError as exc:
             if (isinstance(exc.args[0], str)
                 and exc.args[0].startswith('invalid load key')):
-                raise ChecksumError('Invalid metadata') from None
+                raise ChecksumError('Invalid metadata')
             raise
     else:
         return meta
