@@ -6,14 +6,16 @@ Copyright (C) 2008-2009 Nikolaus Rath <Nikolaus@rath.org>
 This program can be distributed under the terms of the GNU GPLv3.
 '''
 
-from .common import sha256_fh, BUFSIZE
+from .common import BUFSIZE
 from .database import NoSuchRowError
+from .multi_lock import MultiLock
 from .logging import logging # Ensure use of custom logger class
 from collections import OrderedDict
 from contextlib import contextmanager
 from llfuse import lock, lock_released
-from queue import Queue, Empty as QueueEmtpy
+from queue import Queue, Empty as QueueEmpty
 import os
+import hashlib
 import shutil
 import threading
 import time
@@ -107,22 +109,16 @@ class SimpleEvent(object):
 class CacheEntry(object):
     """An element in the block cache
     
-    If `obj_id` is `None`, then the object has not yet been
-    uploaded to the backend. 
-    
     Attributes:
     -----------
     
-    :dirty:
-       entry has been changed since it was last uploaded.
-    
-    :size: current file size
-    
+    :dirty:    entry has been changed since it was last uploaded.
+    :size:     current file size
     :pos: current position in file
     """
 
     __slots__ = [ 'dirty', 'inode', 'blockno', 'last_access',
-                  'size', 'pos', 'fh' ]
+                  'size', 'pos', 'fh', 'removed' ]
 
     def __init__(self, inode, blockno, filename):
         super().__init__()
@@ -178,6 +174,36 @@ class CacheEntry(object):
         return ('<%sCacheEntry, inode=%d, blockno=%d>'
                 % ('Dirty ' if self.dirty else '', self.inode, self.blockno))
 
+
+class CacheDict(OrderedDict):
+    '''
+    An ordered dictionary designed to store CacheEntries.
+
+    Attributes:
+    
+    :max_size: maximum size to which cache can grow
+    :max_entries: maximum number of entries in cache
+    :size: current size of all entries together
+    '''
+
+    def __init__(self, max_size, max_entries):
+        super().__init__()
+        self.max_size = max_size
+        self.max_entries = max_entries
+        self.size = 0
+
+    def remove(self, key):
+        '''Remove *key* from disk and cache, update size'''
+        
+        el = self.pop(key)
+        el.close()
+        el.unlink()
+        self.size -= el.size
+
+    def is_full(self):
+        return (self.size > self.max_size
+                or len(self) > self.max_entries)
+
 class BlockCache(object):
     """Provides access to file blocks
     
@@ -191,25 +217,16 @@ class BlockCache(object):
     ----------
     
     :path: where cached data is stored
-    :entries: ordered dictionary of cache entries
-    :size: current size of all cached entries
-    :max_size: maximum size to which cache can grow
-    :in_transit: set of objects currently in transit and
-         (inode, blockno) tuples currently being uploaded 
-    :removed_in_transit: set of objects that have been removed from the db
-       while in transit, and should be removed from the backend as soon
-       as the transit completes.
+    :cache: ordered dictionary of cache entries
+    :mlock: MultiLock to synchronize access to objects and cache entries
+    :in_transit: set of cache entries that are currently being uploaded
     :to_upload: distributes objects to upload to worker threads
     :to_remove: distributes objects to remove to worker threads
-    :transfer_complete: signals completion of an object transfer
-       (either upload or download)
-    
-    The `in_transit` attribute is used to
-    - Prevent multiple threads from downloading the same object
-    - Prevent threads from downloading an object before it has been
-      uploaded completely (can happen when a cache entry is linked to a
-      block while the object containing the block is still being
-      uploaded)
+    :transfer_complete: signals completion of an object upload
+    :upload_threads: list of threads processing upload queue
+    :removal_threads: list of threads processing removal queue
+    :db: Handle to SQL DB
+    :backend_pool: BackendPool instance
     """
 
     def __init__(self, backend_pool, db, cachedir, max_size, max_entries=768):
@@ -218,12 +235,9 @@ class BlockCache(object):
         self.path = cachedir
         self.db = db
         self.backend_pool = backend_pool
-        self.entries = OrderedDict()
-        self.max_entries = max_entries
-        self.size = 0
-        self.max_size = max_size
+        self.cache = CacheDict(max_size, max_entries)
+        self.mlock = MultiLock()
         self.in_transit = set()
-        self.removed_in_transit = set()
         self.upload_threads = []
         self.removal_threads = []
         self.transfer_completed = SimpleEvent()
@@ -237,7 +251,7 @@ class BlockCache(object):
 
     def __len__(self):
         '''Get number of objects in cache'''
-        return len(self.entries)
+        return len(self.cache)
 
     def init(self, threads=1):
         '''Start worker threads'''
@@ -255,6 +269,43 @@ class BlockCache(object):
             t.start()
             self.removal_threads.append(t)
 
+    def _lock_obj(self, obj_id, release_global=False):
+        '''Acquire lock on *obj*id*'''
+
+        if release_global:
+            with lock_released:
+                self.mlock.acquire(obj_id)
+        else:
+            self.mlock.acquire(obj_id)
+
+    def _unlock_obj(self, obj_id, release_global=False, noerror=False):
+        '''Release lock on *obj*id*'''
+
+        if release_global:
+            with lock_released:
+                self.mlock.release(obj_id, noerror=noerror)
+        else:
+            self.mlock.release(obj_id, noerror=noerror)
+
+    def _lock_entry(self, inode, blockno, release_global=False):
+        '''Acquire lock on cache entry'''
+
+        if release_global:
+            with lock_released:
+                self.mlock.acquire((inode, blockno))
+        else:
+            self.mlock.acquire((inode, blockno))
+
+    def _unlock_entry(self, inode, blockno, release_global=False,
+                      noerror=False):
+        '''Release lock on cache entry'''
+        
+        if release_global:
+            with lock_released:
+                self.mlock.release((inode, blockno), noerror=noerror)
+        else:
+            self.mlock.release((inode, blockno), noerror=noerror)
+            
     def destroy(self):
         '''Clean up and stop worker threads
         
@@ -274,13 +325,21 @@ class BlockCache(object):
         log.debug('destroy(): waiting for upload threads...')
         for t in self.upload_threads:
             t.join()
-        self.to_upload = None
         
         log.debug('destroy(): waiting for removal threads...')
         for t in self.removal_threads:
             t.join()
-        self.to_remove = None
+
+        assert len(self.in_transit) == 0
+        try:
+            self.to_remove.get_nowait()
+        except QueueEmpty:
+            pass
+        else:
+            raise RuntimeError('Internal error: removal queue not empty.')
         
+        self.to_upload = None
+        self.to_remove = None
         self.upload_threads = None
         self.removal_threads = None
 
@@ -288,6 +347,7 @@ class BlockCache(object):
 
         log.debug('cleanup done.')
 
+        
     def _upload_loop(self):
         '''Process upload queue'''
 
@@ -299,17 +359,13 @@ class BlockCache(object):
 
             self._do_upload(*tmp)
 
+            
     def _do_upload(self, el, obj_id):
         '''Upload object'''
 
         def do_write(fh):
-            # If the object has been removed in transit, the cache element is
-            # already closed and we can't seek.
-            if obj_id in self.removed_in_transit:
-                return fh
-            
             el.seek(0)
-            while obj_id not in self.removed_in_transit:
+            while True:
                 buf = el.read(BUFSIZE)
                 if not buf:
                     break
@@ -317,47 +373,32 @@ class BlockCache(object):
             return fh
 
         try:
-            if log.isEnabledFor(logging.DEBUG):
-                time_ = time.time()
-
             with self.backend_pool() as backend:
-                obj_size = backend.perform_write(do_write, 's3ql_data_%d' % obj_id).get_obj_size()
-
-            removed = obj_id in self.removed_in_transit
-            
-            if log.isEnabledFor(logging.DEBUG):
-                if removed:
-                    log.debug('_do_upload(%s): aborted upload of deleted object', obj_id)
-                else:
+                if log.isEnabledFor(logging.DEBUG):
+                    time_ = time.time()
+                    obj_size = backend.perform_write(do_write, 's3ql_data_%d'
+                                                     % obj_id).get_obj_size()
                     time_ = time.time() - time_
                     rate = el.size / (1024 ** 2 * time_) if time_ != 0 else 0
                     log.debug('_do_upload(%s): uploaded %d bytes in %.3f seconds, %.2f MiB/s',
                               obj_id, el.size, time_, rate)
-
-            if removed:
-                self.removed_in_transit.remove(obj_id)
-                self.to_remove.put(obj_id)
+                else:
+                    obj_size = backend.perform_write(do_write, 's3ql_data_%d'
+                                                     % obj_id).get_obj_size()
 
             with lock:
-                if not removed:
-                    self.db.execute('UPDATE objects SET size=? WHERE id=?',
-                                    (obj_size, obj_id))
-
+                self.db.execute('UPDATE objects SET size=? WHERE id=?', (obj_size, obj_id))
                 el.dirty = False
-                self.in_transit.remove(obj_id)
-                self.in_transit.remove((el.inode, el.blockno))
-                self.transfer_completed.notify_all()
 
-        except:
-            with lock:
-                self.in_transit.remove(obj_id)
-                self.in_transit.remove((el.inode, el.blockno))
-                self.transfer_completed.notify_all()
-            raise
+        finally:
+            self.in_transit.remove(el)
+            self._unlock_obj(obj_id)
+            self._unlock_entry(el.inode, el.blockno)
+            self.transfer_completed.notify_all()
 
 
     def wait(self):
-        '''Wait until an object has been transferred
+        '''Wait until an object has been uploaded
         
         If there are no objects in transit, return immediately. This method
         releases the global lock.
@@ -372,20 +413,40 @@ class BlockCache(object):
     def upload(self, el):
         '''Upload cache entry `el` asynchronously
         
-        Return (uncompressed) size of cache entry.
-        
         This method releases the global lock.
         '''
 
         log.debug('upload(%s): start', el)
 
-        assert (el.inode, el.blockno) not in self.in_transit
-        self.in_transit.add((el.inode, el.blockno))
+        # Calculate checksum
+        with lock_released:
+            try:
+                self._lock_entry(el.inode, el.blockno)
+                assert el not in self.in_transit
+                if not el.dirty:
+                    log.debug('upload(%s): not dirty, returning', el)
+                    self._unlock_entry(el.inode, el.blockno)
+                    return
+                if (el.inode, el.blockno) not in self.cache:
+                    log.debug('%s removed while waiting for lock', el)
+                    self._unlock_entry(el.inode, el.blockno)
+                    return
+                
+                self.in_transit.add(el)
+                sha = hashlib.sha256()
+                el.seek(0)
+                while True:
+                    buf = el.read(BUFSIZE)
+                    if not buf:
+                        break
+                    sha.update(buf)
+                hash_ = sha.digest()
+            except:
+                self.in_transit.discard(el)
+                self._unlock_entry(el.inode, el.blockno)
+                raise
 
         try:
-            el.seek(0)
-            hash_ = sha256_fh(el)
-
             try:
                 old_block_id = self.db.get_val('SELECT block_id FROM inode_blocks '
                                                'WHERE inode=? AND blockno=?',
@@ -410,80 +471,101 @@ class BlockCache(object):
                 # are available in db.
                 self.db.execute('INSERT OR REPLACE INTO inode_blocks (block_id, inode, blockno) '
                                 'VALUES(?,?,?)', (block_id, el.inode, el.blockno))
-
-                self.in_transit.add(obj_id)
+                
                 with lock_released:
+                    self._lock_obj(obj_id)
                     self.to_upload.put((el, obj_id))
 
-            # There is a block with the same hash                        
+            # There is a block with the same hash
             else:
+                if old_block_id != block_id:
+                    log.debug('upload(%s): (re)linking to %d', el, block_id)
+                    self.db.execute('UPDATE blocks SET refcount=refcount+1 WHERE id=?',
+                                    (block_id,))
+                    self.db.execute('INSERT OR REPLACE INTO inode_blocks (block_id, inode, blockno) '
+                                    'VALUES(?,?,?)', (block_id, el.inode, el.blockno))
+
+                el.dirty = False
+                self.in_transit.remove(el)
+                self._unlock_entry(el.inode, el.blockno, release_global=True)
+                
                 if old_block_id == block_id:
                     log.debug('upload(%s): unchanged, block_id=%d', el, block_id)
-                    el.dirty = False
-                    self.in_transit.remove((el.inode, el.blockno))
-                    return el.size
-
-                log.debug('upload(%s): (re)linking to %d', el, block_id)
-                self.db.execute('UPDATE blocks SET refcount=refcount+1 WHERE id=?',
-                                (block_id,))
-                self.db.execute('INSERT OR REPLACE INTO inode_blocks (block_id, inode, blockno) '
-                                'VALUES(?,?,?)', (block_id, el.inode, el.blockno))
-                el.dirty = False
-                self.in_transit.remove((el.inode, el.blockno))
+                    return
+                
         except:
-            self.in_transit.remove((el.inode, el.blockno))
+            self.in_transit.discard(el)
+            with lock_released:
+                self._unlock_entry(el.inode, el.blockno, noerror=True)
+                self._unlock_obj(obj_id, noerror=True)
             raise
 
         # Check if we have to remove an old block
         if not old_block_id:
             log.debug('upload(%s): no old block, returning', el)
-            return el.size
+            return
 
-        refcount = self.db.get_val('SELECT refcount FROM blocks WHERE id=?', (old_block_id,))
-        if refcount > 1:
-            log.debug('upload(%s):  decreased refcount for prev. block: %d', el, old_block_id)
-            self.db.execute('UPDATE blocks SET refcount=refcount-1 WHERE id=?', (old_block_id,))
-            return el.size
+        self._deref_block(old_block_id)
 
-        log.debug('upload(%s): removing prev. block %d', el, old_block_id)
-        old_obj_id = self.db.get_val('SELECT obj_id FROM blocks WHERE id=?', (old_block_id,))
-        self.db.execute('DELETE FROM blocks WHERE id=?', (old_block_id,))
-        refcount = self.db.get_val('SELECT refcount FROM objects WHERE id=?', (old_obj_id,))
+    
+    def _deref_block(self, block_id):
+        '''Decrease reference count for *block_id*
+
+        If reference counter drops to zero, remove block and propagate
+        to objects table (possibly removing the referenced object
+        as well).
+
+        This method releases the global lock.
+        '''
+        
+        refcount = self.db.get_val('SELECT refcount FROM blocks WHERE id=?', (block_id,))
         if refcount > 1:
-            log.debug('upload(%s):  decreased refcount for prev. obj: %d', el, old_obj_id)
+            log.debug('decreased refcount for block: %d', block_id)
+            self.db.execute('UPDATE blocks SET refcount=refcount-1 WHERE id=?', (block_id,))
+            return
+
+        log.debug('removing block %d', block_id)
+        obj_id = self.db.get_val('SELECT obj_id FROM blocks WHERE id=?', (block_id,))
+        self.db.execute('DELETE FROM blocks WHERE id=?', (block_id,))
+        refcount = self.db.get_val('SELECT refcount FROM objects WHERE id=?', (obj_id,))
+        if refcount > 1:
+            log.debug('decreased refcount for obj: %d', obj_id)
             self.db.execute('UPDATE objects SET refcount=refcount-1 WHERE id=?',
-                            (old_obj_id,))
-            return el.size
+                            (obj_id,))
+            return
 
-        log.debug('upload(%s): removing object %d', el, old_obj_id)
-        self.db.execute('DELETE FROM objects WHERE id=?', (old_obj_id,))
+        log.debug('removing object %d', obj_id)
+        self.db.execute('DELETE FROM objects WHERE id=?', (obj_id,))
 
-        if old_obj_id in self.in_transit:
-            log.debug('upload(%s): deferring removal of old object %d, still in transit',
-                      el, old_obj_id)
-            self.removed_in_transit.add(old_obj_id)
-        else:
-            log.debug('upload(%s): adding %d to removal queue', el, old_obj_id)
-            with lock_released:
-                self.to_remove.put(old_obj_id)
-
-        return el.size
+        # Taking the lock ensures that the object is no longer in
+        # transit itself. We can release it immediately after, because
+        # the object is no longer in the database.
+        log.debug('adding %d to removal queue', obj_id)
+        
+        with lock_released:
+            self._lock_obj(obj_id)
+            self._unlock_obj(obj_id)
+            self.to_remove.put(obj_id)
 
 
     def transfer_in_progress(self):
-        '''Return True if there are any blocks in transit'''
+        '''Return True if there are any cache entries being uploaded'''
 
         return len(self.in_transit) > 0
 
     def _removal_loop(self):
         '''Process removal queue'''
 
+        # This method may look more complicated than necessary, but
+        # it ensures that we read as many objects from the queue
+        # as we can without blocking, and then hand them over to
+        # the backend all at once.
         ids = []
         while True:
             try:
                 log.debug('reading from queue (blocking=%s)', len(ids)==0)
                 tmp = self.to_remove.get(block=len(ids)==0)
-            except QueueEmtpy:
+            except QueueEmpty:
                 tmp = FlushSentinel
 
             if (tmp is FlushSentinel or tmp is QuitSentinel) and ids:
@@ -502,8 +584,10 @@ class BlockCache(object):
     def get(self, inode, blockno):
         """Get file handle for block `blockno` of `inode`
         
-        This method releases the global lock, and the managed block
-        may do so as well.
+        This method releases the global lock. The managed block, however,
+        is executed with the global lock acquired and MUST NOT release
+        it. This ensures that only one thread is accessing a given block
+        at a time.
         
         Note: if `get` and `remove` are called concurrently, then it is
         possible that a block that has been requested with `get` and
@@ -512,96 +596,84 @@ class BlockCache(object):
 
         #log.debug('get(inode=%d, block=%d): start', inode, blockno)
 
-        if self.size > self.max_size or len(self.entries) > self.max_entries:
+        if self.cache.is_full():
             self.expire()
 
-        el = None
-        while el is None:
-            # Don't allow changing objects while they're being uploaded
-            if (inode, blockno) in self.in_transit:
-                log.debug('get(inode=%d, block=%d): inode/blockno in transit, waiting',
-                          inode, blockno)
-                self.wait()
-                continue
-
-            try:
-                el = self.entries[(inode, blockno)]
-
-            # Not in cache
-            except KeyError:
-                filename = os.path.join(self.path, '%d-%d' % (inode, blockno))
-                try:
-                    block_id = self.db.get_val('SELECT block_id FROM inode_blocks '
-                                               'WHERE inode=? AND blockno=?', (inode, blockno))
-
-                # No corresponding object
-                except NoSuchRowError:
-                    #log.debug('get(inode=%d, block=%d): creating new block', inode, blockno)
-                    el = CacheEntry(inode, blockno, filename)
-                    self.entries[(inode, blockno)] = el
-
-                # Need to download corresponding object
-                else:
-                    #log.debug('get(inode=%d, block=%d): downloading block', inode, blockno)
-                    obj_id = self.db.get_val('SELECT obj_id FROM blocks WHERE id=?', (block_id,))
-
-                    if obj_id in self.in_transit:
-                        log.debug('get(inode=%d, block=%d): object %d in transit, waiting',
-                                  inode, blockno, obj_id)
-                        self.wait()
-                        continue
-
-                    # We need to download
-                    self.in_transit.add(obj_id)
-                    log.debug('get(inode=%d, block=%d): downloading object %d..',
-                              inode, blockno, obj_id)
-                        
-                    el = CacheEntry(inode, blockno, filename)    
-                    try:
-                        def do_read(fh):
-                            el.seek(0)
-                            el.truncate()
-                            shutil.copyfileobj(fh, el, BUFSIZE)
-
-                        with lock_released:
-                            with self.backend_pool() as backend:
-                                backend.perform_read(do_read, 's3ql_data_%d' % obj_id)
-
-                        # Note: We need to do this *before* releasing the global
-                        # lock to notify other threads
-                        self.entries[(inode, blockno)] = el
-
-                        # Writing will have set dirty flag
-                        el.dirty = False
-                        self.size += el.size
-
-                    except:
-                        el.unlink()
-                        el.close()
-                        raise
-                    finally:
-                        self.in_transit.remove(obj_id)
-                        with lock_released:
-                            self.transfer_completed.notify_all()
-
-            # In Cache
-            else:
-                #log.debug('get(inode=%d, block=%d): in cache', inode, blockno)
-                self.entries.move_to_end((inode, blockno), last=True) # move to head
-
-        el.last_access = time.time()
-        oldsize = el.size
-
-        # Provide fh to caller
+        self._lock_entry(inode, blockno, release_global=True)
         try:
-            #log.debug('get(inode=%d, block=%d): yield', inode, blockno)
-            yield el
+            el = self._get_entry(inode, blockno)
+            el.last_access = time.time()
+            oldsize = el.size
+            try:
+                yield el
+            finally:
+                # Update cachesize. NOTE: this requires that at most one
+                # thread has access to a cache entry at any time.
+                self.cache.size += el.size - oldsize
         finally:
-            # Update cachesize 
-            self.size += el.size - oldsize
-
+            self._unlock_entry(inode, blockno, release_global=True)
+            
         #log.debug('get(inode=%d, block=%d): end', inode, blockno)
 
+    def _get_entry(self, inode, blockno):
+        '''Get cache entry for `blockno` of `inode`
+
+        Assume that cache entry lock has been acquired.
+        '''
+        
+        try:
+            el = self.cache[(inode, blockno)]
+
+        # Not in cache
+        except KeyError:
+            filename = os.path.join(self.path, '%d-%d' % (inode, blockno))
+            try:
+                block_id = self.db.get_val('SELECT block_id FROM inode_blocks '
+                                           'WHERE inode=? AND blockno=?', (inode, blockno))
+
+            # No corresponding object
+            except NoSuchRowError:
+                log.debug('_get_entry(inode=%d, block=%d): creating new block', inode, blockno)
+                el = CacheEntry(inode, blockno, filename)
+                self.cache[(inode, blockno)] = el
+                return el
+            
+            # Need to download corresponding object
+            obj_id = self.db.get_val('SELECT obj_id FROM blocks WHERE id=?', (block_id,))
+            log.debug('_get_entry(inode=%d, block=%d): downloading object %d..',
+                      inode, blockno, obj_id)
+            el = CacheEntry(inode, blockno, filename)
+            try: 
+                def do_read(fh):
+                    el.seek(0)
+                    el.truncate()
+                    shutil.copyfileobj(fh, el, BUFSIZE)
+
+                with lock_released:
+                    # Lock object. This ensures that we wait until the object
+                    # is uploaded. We don't have to worry about deletion, because
+                    # as long as the current cache entry exists, there will always be
+                    # a reference to the object (and we already have a lock on the
+                    # cache entry).
+                    self._lock_obj(obj_id)
+                    self._unlock_obj(obj_id)
+                    with self.backend_pool() as backend:
+                        backend.perform_read(do_read, 's3ql_data_%d' % obj_id)
+            except:
+                el.unlink()
+                el.close()
+                raise
+            
+            self.cache[(inode, blockno)] = el
+            el.dirty = False # (writing will have set dirty flag)
+            self.cache.size += el.size
+
+        # In Cache
+        else:
+            #log.debug('_get_entry(inode=%d, block=%d): in cache', inode, blockno)
+            self.cache.move_to_end((inode, blockno), last=True) # move to head
+            
+        return el
 
     def expire(self):
         """Perform cache expiry
@@ -614,60 +686,48 @@ class BlockCache(object):
 
         log.debug('expire: start')
 
-        did_nothing_count = 0
-        while (len(self.entries) > self.max_entries or
-               (len(self.entries) > 0  and self.size > self.max_size)):
-
-            need_size = self.size - self.max_size
-            need_entries = len(self.entries) - self.max_entries
-
-            # Try to expire entries that are not dirty
-            for el in self.entries.values():
-                if el.dirty:
-                    if (el.inode, el.blockno) in self.in_transit:
-                        log.debug('expire: %s is dirty, but already being uploaded', el)
-                        continue
-                    else:
-                        log.debug('expire: %s is dirty, trying to flush', el)
-                        break
-
-                del self.entries[(el.inode, el.blockno)]
-                el.close()
-                el.unlink()
-                need_entries -= 1
-                self.size -= el.size
-                need_size -= el.size
-
-                did_nothing_count = 0
-                if need_size <= 0 and need_entries <= 0:
-                    break
-
+        while True:
+            need_size = self.cache.size - self.cache.max_size
+            need_entries = len(self.cache) - self.cache.max_entries
+            
             if need_size <= 0 and need_entries <= 0:
                 break
 
-            # Try to upload just enough
-            for el in self.entries.values():
-                if el.dirty and (el.inode, el.blockno) not in self.in_transit:
-                    log.debug('expire: uploading %s..', el)
-                    freed = self.upload(el) # Releases global lock
-                    need_size -= freed
-                    did_nothing_count = 0
-                else:
-                    log.debug('expire: %s can soon be expired..', el)
-                    need_size -= el.size
-                need_entries -= 1
-
+            # Need to make copy, since we aren't allowed to change dict while
+            # iterating through it. Look at the comments in CommitThread.run()
+            # (mount.py) for an estimate of the resulting performance hit.
+            sth_in_transit = False
+            for el in list(self.cache.values()):
                 if need_size <= 0 and need_entries <= 0:
                     break
+            
+                need_entries -= 1
+                need_size -= el.size
 
-            did_nothing_count += 1
-            if did_nothing_count > 50:
-                log.error('Problem in expire()')
-                break
+                if el.dirty:
+                    if el not in self.in_transit:
+                        log.debug('expire: uploading %s..', el)
+                        self.upload(el) # Releases global lock
+                    sth_in_transit = True
+                    continue
 
-            # Wait for the next entry  
-            log.debug('expire: waiting for transfer threads..')
-            self.wait() # Releases global lock
+                log.debug('removing inode %d, block %d from cache', el.inode, el.blockno)
+                self._lock_entry(el.inode, el.blockno, release_global=True)
+                try:
+                    # May have changed while we were waiting for lock
+                    if el.dirty:
+                        log.debug('%s got dirty while waiting for lock', el)
+                        continue
+                    if (el.inode, el.blockno) not in self.cache:
+                        log.debug('%s removed while waiting for lock', el)
+                        continue
+                    self.cache.remove((el.inode, el.blockno))
+                finally:
+                    self._unlock_entry(el.inode, el.blockno, release_global=True)
+
+            if sth_in_transit:
+                log.debug('expire: waiting for transfer threads..')
+                self.wait() # Releases global lock
 
         log.debug('expire: end')
 
@@ -680,10 +740,6 @@ class BlockCache(object):
          `end_no`. 
         
         This method releases the global lock.
-        
-        Note: if `get` and `remove` are called concurrently, then it is possible
-        that a block that has been requested with `get` and passed to `remove`
-        for deletion will not be deleted.
         """
 
         log.debug('remove(inode=%d, start=%d, end=%s): start', inode, start_no, end_no)
@@ -692,70 +748,41 @@ class BlockCache(object):
             end_no = start_no + 1
 
         for blockno in range(start_no, end_no):
-            if (inode, blockno) in self.entries:
-                log.debug('remove(inode=%d, blockno=%d): removing from cache',
-                          inode, blockno)
-
-                # Type inference fails here
-                #pylint: disable-msg=E1103
-                el = self.entries.pop((inode, blockno))
-
-                self.size -= el.size
-                el.unlink()
-                el.close()
-
+            self._lock_entry(inode, blockno, release_global=True)
             try:
-                block_id = self.db.get_val('SELECT block_id FROM inode_blocks '
-                                           'WHERE inode=? AND blockno=?', (inode, blockno))
-            except NoSuchRowError:
-                log.debug('remove(inode=%d, blockno=%d): block not in db', inode, blockno)
-                continue
+                if (inode, blockno) in self.cache:
+                    log.debug('remove(inode=%d, blockno=%d): removing from cache',
+                              inode, blockno)
+                    self.cache.remove((inode, blockno))
 
-            # Detach inode from block
-            self.db.execute('DELETE FROM inode_blocks WHERE inode=? AND blockno=?',
-                            (inode, blockno))
+                try:
+                    block_id = self.db.get_val('SELECT block_id FROM inode_blocks '
+                                               'WHERE inode=? AND blockno=?', (inode, blockno))
+                except NoSuchRowError:
+                    log.debug('remove(inode=%d, blockno=%d): block not in db', inode, blockno)
+                    continue
 
+                # Detach inode from block
+                self.db.execute('DELETE FROM inode_blocks WHERE inode=? AND blockno=?',
+                                (inode, blockno))
+
+            finally:
+                self._unlock_entry(inode, blockno, release_global=True)
+                
             # Decrease block refcount
-            refcount = self.db.get_val('SELECT refcount FROM blocks WHERE id=?', (block_id,))
-            if refcount > 1:
-                log.debug('remove(inode=%d, blockno=%d): decreasing refcount for block %d',
-                          inode, blockno, block_id)
-                self.db.execute('UPDATE blocks SET refcount=refcount-1 WHERE id=?',
-                                (block_id,))
-                continue
-
-            # Detach block from object
-            log.debug('remove(inode=%d, blockno=%d): deleting block %d',
-                      inode, blockno, block_id)
-            obj_id = self.db.get_val('SELECT obj_id FROM blocks WHERE id=?', (block_id,))
-            self.db.execute('DELETE FROM blocks WHERE id=?', (block_id,))
-
-            # Decrease object refcount
-            refcount = self.db.get_val('SELECT refcount FROM objects WHERE id=?', (obj_id,))
-            if refcount > 1:
-                log.debug('remove(inode=%d, blockno=%d): decreasing refcount for object %d',
-                          inode, blockno, obj_id)
-                self.db.execute('UPDATE objects SET refcount=refcount-1 WHERE id=?',
-                                (obj_id,))
-            else:
-                if obj_id in self.in_transit:
-                    log.debug('remove(inode=%d, blockno=%d): deferring removal of '
-                              'object %d, still in transit', inode, blockno, obj_id)
-                    self.removed_in_transit.add(obj_id)
-                else:
-                    log.debug('remove(inode=%d, blockno=%d): deleting object %d',
-                              inode, blockno, obj_id)
-                    with lock_released:
-                        self.to_remove.put(obj_id)
-                self.db.execute('DELETE FROM objects WHERE id=?', (obj_id,))
+            self._deref_block(block_id)
 
         log.debug('remove(inode=%d, start=%d, end=%s): end', inode, start_no, end_no)
 
-    def flush(self, inode):
-        """Flush buffers for `inode`"""
+    def flush(self, inode, blockno):
+        """Flush buffers for given block"""
 
-        # Cache entries are automatically flushed after each read() and write()
-        pass
+        try:
+            el = self.cache[(inode, blockno)]
+        except KeyError:
+            return
+
+        el.flush()
 
     def commit(self):
         """Initiate upload of all dirty blocks
@@ -767,8 +794,11 @@ class BlockCache(object):
         This method releases the global lock.
         """
 
-        for el in self.entries.values():
-            if not (el.dirty and (el.inode, el.blockno) not in self.in_transit):
+        # Need to make copy, since we aren't allowed to change dict while
+        # iterating through it. Look at the comments in CommitThread.run()
+        # (mount.py) for an estimate of the resulting performance hit.
+        for el in list(self.cache.values()):
+            if not el.dirty or el in self.in_transit:
                 continue
 
             self.upload(el) # Releases global lock
@@ -780,15 +810,15 @@ class BlockCache(object):
         """
 
         log.debug('clear: start')
-        bak = self.max_entries
-        self.max_entries = 0
+        bak = self.cache.max_entries
+        self.cache.max_entries = 0
         self.expire() # Releases global lock
-        self.max_entries = bak
+        self.cache.max_entries = bak
 
         log.debug('clear: end')
 
 
     def __del__(self):
-        if len(self.entries) > 0:
-            raise RuntimeError("BlockCache instance was destroyed without calling destroy()!")
+        if len(self.cache) > 0:
+            raise RuntimeError("BlockManager instance was destroyed without calling destroy()!")
 
