@@ -9,6 +9,7 @@ This program can be distributed under the terms of the GNU GPLv3.
 
 from .logging import logging, QuietError # Ensure use of custom logger class
 from llfuse import ROOT_INODE
+import subprocess
 import bz2
 import errno
 import hashlib
@@ -100,6 +101,34 @@ def stream_read_bz2(ifh, ofh):
     if decompressor.unused_data or ifh.read(1) != b'':
         raise ChecksumError('Data after end of bz2 stream')
 
+    
+def is_mounted(storage_url):
+    '''Try to determine if *storage_url* is mounted
+
+    Note that the result may be wrong.. this is really just
+    a best-effort guess.
+    '''
+
+    match = storage_url + ' '
+    if os.path.exists('/proc/mounts'):
+        with open('/proc/mounts', 'r') as fh:
+            for line in fh:
+                if line.startswith(match):
+                    return True
+            return False
+
+    try:
+        for line in subprocess.check_output(['mount'], stderr=subprocess.STDOUT,
+                                            universal_newlines=True):
+            if line.startswith(match):
+                return True
+    except subprocess.CalledProcessError:
+        log.warning('Warning! Unable to check if file system is mounted '
+                    '(/proc/mounts missing and mount call failed)')
+    
+    return False
+
+        
 class ChecksumError(Exception):
     """
     Raised if there is a checksum error in the data that we received.
@@ -132,7 +161,7 @@ def inode_for_path(path, conn):
             inode = conn.get_val("SELECT inode FROM contents_v WHERE name=? AND parent_inode=?",
                                  (el, inode))
         except NoSuchRowError:
-            raise KeyError('Path %s does not exist' % path) from None
+            raise KeyError('Path %s does not exist' % path)
 
     return inode
 
@@ -182,7 +211,7 @@ def get_backend_cachedir(storage_url, cachedir):
         try:
             os.mkdir(cachedir, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)
         except PermissionError:
-            raise QuietError('No permission to create cache directory (%s)' % cachedir) from None
+            raise QuietError('No permission to create cache directory (%s)' % cachedir)
         
     if not os.access(cachedir, os.R_OK | os.W_OK | os.X_OK):
         raise QuietError('No permission to access cache directory (%s)' % cachedir)
@@ -224,10 +253,10 @@ def assert_s3ql_fs(path):
     try:
         os.stat(path)
     except FileNotFoundError:
-        raise QuietError('%s does not exist' % path) from None
+        raise QuietError('%s does not exist' % path)
     except OSError as exc:
         if exc.errno is errno.ENOTCONN:
-            raise QuietError('File system appears to have crashed.') from None
+            raise QuietError('File system appears to have crashed.')
         raise
 
     ctrlfile = os.path.join(path, CTRL_NAME)
