@@ -98,7 +98,7 @@ def main(args=None):
         plain_backend = get_backend(options, plain=True)
         atexit.register(plain_backend.close)
     except DanglingStorageURLError as exc:
-        raise QuietError(str(exc)) from None
+        raise QuietError(str(exc))
 
     log.info("Before using S3QL, make sure to read the user's guide, especially\n"
              "the 'Important Rules to Avoid Loosing Data' section.")
@@ -135,6 +135,9 @@ def main(args=None):
 
         backend = BetterBackend(wrap_pw, ('lzma', 2), plain_backend)
         backend['s3ql_passphrase'] = data_pw
+        backend['s3ql_passphrase_bak1'] = data_pw
+        backend['s3ql_passphrase_bak2'] = data_pw
+        backend['s3ql_passphrase_bak3'] = data_pw
     else:
         data_pw = None
 
