@@ -17,20 +17,24 @@ from .database import Connection
 from .inode_cache import InodeCache
 from .metadata import cycle_metadata, dump_metadata, restore_metadata
 from .parse_args import ArgumentParser
+from .exit_stack import ExitStack
 from threading import Thread
-from contextlib import ExitStack
 import _thread
 import argparse
 import faulthandler
 import llfuse
 import os
+import platform
+import subprocess
 import pickle
 import re
 import signal
+import resource
 import sys
 import tempfile
 import threading
 import time
+import shutil
 import atexit
 
 log = logging.getLogger(__name__)
@@ -72,11 +76,33 @@ def main(args=None):
     # Save handler so that we can remove it when daemonizing
     stdout_log_handler = setup_logging(options)
 
+    if not os.path.exists(options.mountpoint):
+        raise QuietError('Mountpoint does not exist.')
+        
     if options.threads is None:
         options.threads = determine_threads(options)
 
-    if not os.path.exists(options.mountpoint):
-        raise QuietError('Mountpoint does not exist.')
+    avail_fd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
+    if avail_fd == resource.RLIM_INFINITY:
+        avail_fd = 4096
+    resource.setrlimit(resource.RLIMIT_NOFILE, (avail_fd, avail_fd))
+
+    # Subtract some fd's for random things we forgot, and a fixed number for
+    # each upload thread (because each thread is using at least one socket and
+    # at least one temporary file)
+    avail_fd -= 32 + 3 * options.threads
+    
+    if options.max_cache_entries is None:
+        if avail_fd <= 64:
+            raise QuietError("Not enough available file descriptors.")
+        log.info('Autodetected %d file descriptors available for cache entries',
+                 avail_fd)
+        options.max_cache_entries = avail_fd
+    else:
+        if options.max_cache_entries > avail_fd:
+            log.warning("Up to %d cache entries requested, but detected only %d "
+                        "available file descriptors.", options.max_cache_entries, avail_fd)
+            options.max_cache_entries = avail_fd
 
     if options.profile:
         import cProfile
@@ -95,11 +121,21 @@ def main(args=None):
         with backend_pool() as backend:
             (param, db) = get_metadata(backend, cachepath)
     except DanglingStorageURLError as exc:
-        raise QuietError(str(exc)) from None
+        raise QuietError(str(exc))
 
     if param['max_obj_size'] < options.min_obj_size:
         raise QuietError('Maximum object size must be bigger than minimum object size.')
 
+    # Handle --cachesize
+    rec_cachesize = options.max_cache_entries * param['max_obj_size'] / 2
+    avail_cache = shutil.disk_usage(os.path.dirname(cachepath))[2] / 1024
+    if options.cachesize is None:
+        options.cachesize = min(rec_cachesize, 0.8 * avail_cache)
+        log.info('Setting cache size to %d MB', options.cachesize / 1024)
+    elif options.cachesize > avail_cache:
+        log.warning('Warning! Requested cache size %d MB, but only %d MB available',
+                    options.cachesize / 1024, avail_cache / 1024)
+        
     if options.nfs:
         # NFS may try to look up '..', so we have to speed up this kind of query
         log.info('Creating NFS indices...')
@@ -122,7 +158,7 @@ def main(args=None):
         try:
             llfuse.init(operations, options.mountpoint, get_fuse_opts(options))
         except RuntimeError as exc:
-            raise QuietError(str(exc)) from None
+            raise QuietError(str(exc))
         
         unmount_clean = False
         def unmount():
@@ -244,15 +280,53 @@ def main(args=None):
 
     log.info('All done.')
 
+
+def get_system_memory():
+    '''Attempt to determine total system memory
+
+    If amount cannot be determined, emits warning and
+    returns -1.
+    '''
+    
+    # MacOS X doesn't support sysconf('SC_PHYS_PAGES')
+    if platform.system() == 'Darwin':
+        try:
+            out = subprocess.check_output(['sysctl', 'hw.memsize'],
+                                          universal_newlines=True)
+        except subprocess.CalledProcessError as exc:
+            log.warning('Cannot determine system memory, sysctl failed with %s',
+                        exc.output)
+            return -1
+
+        # output of sysctl is 'hw.memsize: #'. Strip the prefix.
+        hit = re.match(r'^hw.memsize: ([0-9]+)$', out)
+        if not hit:
+            log.warning('Cannot determine system memory, unable to parse sysctl output.')
+            return -1
+            
+        return int(hit.group(1))
+        
+    else:
+        try:
+            return os.sysconf('SC_PHYS_PAGES') * os.sysconf('SC_PAGESIZE')
+        except ValueError:
+            log.warning('Unable to determine number of CPU cores (sysconf failed).')
+            return -1
+
+            
 # Memory required for LZMA compression in MB (from xz(1))
 LZMA_MEMORY = { 0: 3, 1: 9, 2: 17, 3: 32, 4: 48,
                 5: 94, 6: 94, 7: 186, 8: 370, 9: 674 }
-
 def determine_threads(options):
     '''Return optimum number of upload threads'''
 
-    cores = os.sysconf('SC_NPROCESSORS_ONLN')
-    memory = os.sysconf('SC_PHYS_PAGES') * os.sysconf('SC_PAGESIZE')
+    try:
+        cores = os.sysconf('SC_NPROCESSORS_ONLN')
+    except ValueError:
+        log.warning('Unable to determine number of CPU cores (sysconf failed).')
+        cores = -1
+
+    memory = get_system_memory()
 
     if options.compress[0] == 'lzma':
         # Keep this in sync with compression level in backends/common.py
@@ -262,9 +336,12 @@ def determine_threads(options):
         # Only check LZMA memory usage
         mem_per_thread = 0
 
-    if cores == -1 or memory == -1:
+    if cores == -1:
         log.warning("Can't determine number of cores, using 2 upload threads.")
-        return 1
+        return 2
+    elif memory == -1 and mem_per_thread != 0:
+        log.warning("Can't determine available memory, using 2 upload threads.")
+        return 2
     elif 2 * cores * mem_per_thread > (memory / 2):
         threads = min(int((memory / 2) // mem_per_thread), 10)
         if threads > 0:
@@ -360,6 +437,11 @@ def get_fuse_opts(options):
 
     fuse_opts = [ "nonempty", 'fsname=%s' % options.storage_url,
                   'subtype=s3ql' ]
+    
+    if platform.system() == 'Darwin':
+        # FUSE4X and OSXFUSE claim to support nonempty, but
+        # neither of them actually do.
+        fuse_opts.remove('nonempty')
 
     if options.allow_other:
         fuse_opts.append('allow_other')
@@ -430,12 +512,10 @@ def parse_args(args):
 
     parser.add_argument("mountpoint", metavar='<mountpoint>', type=os.path.abspath,
                         help='Where to mount the file system')
-    parser.add_argument("--cachesize", type=int, default=102400, metavar='<size>',
-                      help="Cache size in KiB (default: 102400 (100 MiB)). Should be at least 10 times "
-                      "the maximum object size of the filesystem, otherwise an object may be retrieved "
-                      "and written several times during a single write() or read() operation.")
-    parser.add_argument("--max-cache-entries", type=int, default=768, metavar='<num>',
-                      help="Maximum number of entries in cache (default: %(default)d). "
+    parser.add_argument("--cachesize", type=int, default=None, metavar='<size>',
+                      help="Cache size in KiB (default: autodetect).")
+    parser.add_argument("--max-cache-entries", type=int, default=None, metavar='<num>',
+                      help="Maximum number of entries in cache (default: autodetect). "
                       'Each cache entry requires one file descriptor, so if you increase '
                       'this number you have to make sure that your process file descriptor '
                       'limit (as set with `ulimit -n`) is high enough (at least the number '
@@ -602,6 +682,7 @@ def setup_exchook():
                 log.warning("Unhandled top-level exception during shutdown "
                             "(will not be re-raised)")
             else:
+                log.debug('recording exception %s', exc_inst)
                 os.kill(os.getpid(), signal.SIGTERM)
                 exc_info.append(exc_inst)
                 exc_info.append(tb)
@@ -609,7 +690,9 @@ def setup_exchook():
 
         # If the main thread re-raised exception, there is no need to call
         # excepthook again
-        elif not exc_info or exc_info[0] is not exc_inst:
+        elif exc_info and exc_info[0] is exc_inst:
+            log.debug('Suppressing exception hook for re-raised %s', exc_inst)
+        else:
             old_exchook(exc_type, exc_inst, tb)
 
     sys.excepthook = exchook
@@ -617,7 +700,6 @@ def setup_exchook():
     return exc_info
 
 
-
 class CommitThread(Thread):
     '''
     Periodically upload dirty blocks.
@@ -641,12 +723,24 @@ class CommitThread(Thread):
             
             with llfuse.lock:
                 stamp = time.time()
-                for el in self.block_cache.entries.values():
+                # Need to make copy, since we aren't allowed to change
+                # dict while iterating through it. The performance hit doesn't seem
+                # to be that bad:
+                # >>> from timeit import timeit
+                # >>> timeit("k=0\nfor el in list(d.values()):\n k += el",
+                # ... setup='\nfrom collections import OrderedDict\nd = OrderedDict()\nfor i in range(5000):\n d[i]=i\n',
+                # ... number=500)/500 * 1e3
+                # 1.3769531380003173
+                # >>> timeit("k=0\nfor el in d.values(n:\n k += el",
+                # ... setup='\nfrom collections import OrderedDict\nd = OrderedDict()\nfor i in range(5000):\n d[i]=i\n',
+                # ... number=500)/500 * 1e3
+                # 1.456586996000624
+                for el in list(self.block_cache.cache.values()):
                     if self.stop_event.is_set():
                         break
                     if stamp - el.last_access < 10:
                         break
-                    if not el.dirty or (el.inode, el.blockno) in self.block_cache.in_transit:
+                    if not el.dirty or el in self.block_cache.in_transit:
                         continue
 
                     self.block_cache.upload(el)
@@ -663,5 +757,6 @@ class CommitThread(Thread):
         log.debug('CommitThread: stop() called')
         self.stop_event.set()
 
+
 if __name__ == '__main__':
     main(sys.argv[1:])
