@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 '''
 t1_backends.py - this file is part of S3QL (http://s3ql.googlecode.com)
 
@@ -6,18 +7,25 @@ Copyright (C) 2008-2009 Nikolaus Rath <Nikolaus@rath.org>
 This program can be distributed under the terms of the GNU GPLv3.
 '''
 
+if __name__ == '__main__':
+    import pytest
+    import sys
+    sys.exit(pytest.main([__file__] + sys.argv[1:]))
+
 from s3ql.backends import local, s3, gs, s3c, swift, rackspace, swiftks
 from s3ql.backends.common import (ChecksumError, ObjectNotEncrypted, NoSuchObject,
     BetterBackend, get_ssl_context, AuthenticationError, AuthorizationError,
-    DanglingStorageURLError, MalformedObjectError)
+    DanglingStorageURLError, MalformedObjectError, DecryptFilter, DecompressFilter)
+from s3ql.common import BUFSIZE
+from common import get_remote_test_info
+import s3ql.backends.common
 from argparse import Namespace
-import configparser
 import os
-import stat
 import tempfile
 import time
 import unittest
 import pytest
+import struct
 
 RETRY_DELAY = 1
 
@@ -67,6 +75,40 @@ class BackendTestsMixin(object):
         self.assertEqual(self.backend[key], value)
         self.assertEqual(self.backend.lookup(key), metadata)
 
+
+    def test_readslowly(self):
+        key = self.newname()
+        value = self.newvalue()
+        metadata = { 'jimmy': 'jups@42' }
+
+        with self.backend.open_write(key, metadata) as fh:
+            fh.write(value)
+
+        def read():
+            buf = []
+            with self.backend.open_read(key) as fh:
+                # Force slow reading from underlying layer
+                if isinstance(fh, (DecryptFilter, DecompressFilter)):
+                    real_read = fh.fh.read
+                    def read_slowly(size):
+                        return real_read(1)
+                    fh.fh.read = read_slowly
+                while True:
+                    buf.append(fh.read(1))
+                    if not buf[-1]:
+                        break
+                    
+                return (b''.join(buf), fh.metadata)
+
+        try:
+            s3ql.backends.common.BUFSIZE = 1
+            (value2, metadata2) = self.retry(read, NoSuchObject)
+        finally:
+            s3ql.backends.common.BUFSIZE = BUFSIZE
+
+        self.assertEqual(value, value2)
+        self.assertEqual(metadata, metadata2)
+        
     def test_setitem(self):
         key = self.newname()
         value = self.newvalue()
@@ -250,33 +292,6 @@ class BackendTestsMixin(object):
         self.retry(lambda: self.assertEqual(self.backend[key2], value),
                    NoSuchObject)
 
-def get_remote_test_info(name, skipTest):
-        authfile = os.path.expanduser('~/.s3ql/authinfo2')
-        if not os.path.exists(authfile):
-            skipTest('No authentication file found.')
-
-        mode = os.stat(authfile).st_mode
-        if mode & (stat.S_IRGRP | stat.S_IROTH):
-            skipTest("Authentication file has insecure permissions")
-
-        config = configparser.ConfigParser()
-        config.read(authfile)
-
-        try:
-            fs_name = config.get(name, 'test-fs')
-            backend_login = config.get(name, 'backend-login')
-            backend_password = config.get(name, 'backend-password')
-        except (configparser.NoOptionError, configparser.NoSectionError):
-            skipTest("Authentication file does not have %s section" % name)
-
-        # Append prefix to make sure that we're starting with an empty bucket
-        if fs_name[-1] != '/':
-            fs_name += '/'
-        fs_name += 's3ql_test_%d/' % time.time()
-
-        return (backend_login, backend_password, fs_name)
-
-
 class S3Tests(BackendTestsMixin, unittest.TestCase):
         
     def setUp(self):
@@ -475,7 +490,29 @@ class LzmaCompressionTests(CompressionTestsMixin, unittest.TestCase):
 class EncryptionTests(CompressionTestsMixin, unittest.TestCase):
 
     def _wrap_backend(self):
-        return BetterBackend(b'schluz', ('zlib', 0), self.plain_backend)
+        return BetterBackend(b'schluz', (None, 0), self.plain_backend)
+
+    def test_multi_packet(self):
+        with self.backend.open_write('my_obj') as fh:
+            for i in range(5):
+                fh.write(b'\xFF' * BUFSIZE)
+
+        with self.backend.open_read('my_obj') as fh:
+            while True:
+                buf = fh.read(BUFSIZE//2)
+                if not buf:
+                    break
+
+    def test_issue431(self):
+        hdr_len = struct.calcsize(b'<I')
+        with self.backend.open_write('my_obj') as fh:
+            fh.write(b'\xFF' * 50)
+            fh.write(b'\xFF' * 50)
+
+        with self.backend.open_read('my_obj') as fh:
+            fh.read(50 + 2*hdr_len)
+            fh.read(50)
+            assert fh.read(50) == b''
 
     def test_corruption(self):
         key = self._make_corrupt_obj()
@@ -487,7 +524,7 @@ class EncryptionTests(CompressionTestsMixin, unittest.TestCase):
         key = self._make_extended_obj()
         with pytest.raises(ChecksumError) as exc:
             self.backend.fetch(key)
-        assert exc.value.str == 'HMAC mismatch'
+        assert exc.value.str == 'Extraneous data at end of object'
 
     def test_encryption(self):
 
@@ -522,3 +559,10 @@ class EncryptionCompressionTests(EncryptionTests):
 
     def _wrap_backend(self):
         return BetterBackend(b'schlurz', ('zlib', 6), self.plain_backend)
+
+
+if __name__ == '__main__':
+    t = EncryptionTests()
+    t.setUp()
+    t.test_readslowly()
+    t.tearDown()
