@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 '''
 t3_fs_api.py - this file is part of S3QL (http://s3ql.googlecode.com)
 
@@ -6,6 +7,11 @@ Copyright (C) 2008-2009 Nikolaus Rath <Nikolaus@rath.org>
 This program can be distributed under the terms of the GNU GPLv3.
 '''
 
+if __name__ == '__main__':
+    import pytest
+    import sys
+    sys.exit(pytest.main([__file__] + sys.argv[1:]))
+
 from llfuse import FUSEError
 from random import randint
 from s3ql import fs
