@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 '''
 t4_adm.py - this file is part of S3QL (http://s3ql.googlecode.com)
 
@@ -6,6 +7,11 @@ Copyright (C) 2008-2009 Nikolaus Rath <Nikolaus@rath.org>
 This program can be distributed under the terms of the GNU GPLv3.
 '''
 
+if __name__ == '__main__':
+    import pytest
+    import sys
+    sys.exit(pytest.main([__file__] + sys.argv[1:]))
+
 from s3ql.backends import local
 from s3ql.backends.common import BetterBackend
 import shutil
@@ -53,7 +59,7 @@ class AdmTests(unittest.TestCase):
         passphrase_new = 'sd982jhd'
 
         proc = subprocess.Popen([sys.executable, os.path.join(BASEDIR, 'bin', 's3qladm'),
-                                 '--quiet', '--fatal-warnings', '--authfile',
+                                 '--quiet', '--fatal-warnings', '--log', 'none', '--authfile',
                                  '/dev/null', 'passphrase', self.storage_url ],
                                 stdin=subprocess.PIPE, universal_newlines=True)
 
@@ -86,8 +92,8 @@ class AdmTests(unittest.TestCase):
 
             proc = subprocess.Popen([sys.executable, os.path.join(BASEDIR, 'bin', 'fsck.s3ql'),
                                      '--quiet', '--fatal-warnings', '--authfile', fh.name,
-                                     self.storage_url ], stdin=subprocess.PIPE,
-                                    universal_newlines=True)
+                                     '--cachedir', self.cache_dir, '--log', 'none', self.storage_url ],
+                                    stdin=subprocess.PIPE, universal_newlines=True)
 
             proc.stdin.close()
             self.assertEqual(proc.wait(), 0)
