@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 '''
 t5_cp.py - this file is part of S3QL (http://s3ql.googlecode.com)
 
@@ -6,10 +7,16 @@ Copyright (C) 2008-2009 Nikolaus Rath <Nikolaus@rath.org>
 This program can be distributed under the terms of the GNU GPLv3.
 '''
 
-from t4_fuse import populate_dir, skip_without_rsync
+if __name__ == '__main__':
+    import pytest
+    import sys
+    sys.exit(pytest.main([__file__] + sys.argv[1:]))
+
+from common import populate_dir, skip_without_rsync, BASEDIR
 import os.path
 import shutil
 import subprocess
+from subprocess import check_output, CalledProcessError
 import t4_fuse
 import sys
 import tempfile
@@ -35,20 +42,20 @@ class cpTests(t4_fuse.fuse_tests):
                                    os.path.join(self.mnt_dir, 'orig') + '/'])
 
             # copy
-            subprocess.check_call([sys.executable, os.path.join(t4_fuse.BASEDIR, 'bin', 's3qlcp'),
+            subprocess.check_call([sys.executable, os.path.join(BASEDIR, 'bin', 's3qlcp'),
                                    '--quiet', os.path.join(self.mnt_dir, 'orig'),
                                    os.path.join(self.mnt_dir, 'copy')])
 
             # compare
-            with subprocess.Popen(['rsync', '-anciHAX', '--delete',
-                                   tempdir + '/', os.path.join(self.mnt_dir, 'copy') + '/'],
-                                  stdout=subprocess.PIPE, universal_newlines=True,
-                                  stderr=subprocess.STDOUT) as rsync:
-                out = rsync.communicate()[0]
+            try:
+                out = check_output(['rsync', '-anciHAX', '--delete', tempdir + '/',
+                                    os.path.join(self.mnt_dir, 'copy') + '/'],
+                                   universal_newlines=True, stderr=subprocess.STDOUT)
+            except CalledProcessError as exc:
+                self.fail('rsync failed with ' + exc.output)
+                
             if out:
                 self.fail('Copy not equal to original, rsync says:\n' + out)
-            elif rsync.returncode != 0:
-                self.fail('rsync failed with ' + out)
 
         finally:
             shutil.rmtree(tempdir)
