@@ -1,16 +1,23 @@
+#!/usr/bin/env python3
 '''
-t5_full.py - this file is part of S3QL (http://s3ql.googlecode.com)
+t5_fsck.py - this file is part of S3QL (http://s3ql.googlecode.com)
 
 Copyright (C) Nikolaus Rath <Nikolaus@rath.org>
 
 This program can be distributed under the terms of the GNU GPLv3.
 '''
 
+if __name__ == '__main__':
+    import pytest
+    import sys
+    sys.exit(pytest.main([__file__] + sys.argv[1:]))
+
 from s3ql.common import get_backend_cachedir
 from s3ql.database import Connection
-from t4_fuse import populate_dir, skip_without_rsync
+from common import populate_dir, skip_without_rsync
 import shutil
 import subprocess
+from subprocess import check_output, CalledProcessError
 import t4_fuse
 import tempfile
 
@@ -52,16 +59,14 @@ class FsckTests(t4_fuse.fuse_tests):
 
             # Compare
             self.mount()
-            with subprocess.Popen(['rsync', '-anciHAX', '--delete',
-                                   '--exclude', '/lost+found',
-                                   ref_dir + '/', self.mnt_dir + '/'],
-                                  stdout=subprocess.PIPE, universal_newlines=True,
-                                  stderr=subprocess.STDOUT) as rsync:
-                out = rsync.communicate()[0]
+            try:
+                out = check_output(['rsync', '-anciHAX', '--delete', '--exclude', '/lost+found',
+                                   ref_dir + '/', self.mnt_dir + '/'], universal_newlines=True,
+                                  stderr=subprocess.STDOUT)
+            except CalledProcessError as exc:
+                self.fail('rsync failed with ' + exc.output)
             if out:
                 self.fail('Copy not equal to original, rsync says:\n' + out)
-            elif rsync.returncode != 0:
-                self.fail('rsync failed with ' + out)
 
             self.umount()
         finally:
